/*!
 * @file main.c
 * @brief HBridge8 Click example
 *
 * # Description
 * This library contains an API for the H-Bridge 8 Click driver.
 * This demo application shows the use of a H-Bridge 8 Click board™.
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Initialization of PWM module and log UART.
 * After driver initialization, the app set duty cycle, start PWM and
 * set motor drive the forward.
 *
 * ## Application Task
 * This is an example that shows the use of an H-Bridge 8 Click board™.
 * In this example, the app drives the motor forward/reverse and switched the  PWM signal back and forth 
 * from 10% duty cycle to 90% duty cycle every 500 milliseconds and back.
 * Results are being sent to the Usart Terminal where you can track their changes.
 *
 * @author Nenad Filipovic
 *
 */

#include "board.h"
#include "log.h"
#include "hbridge8.h"

static hbridge8_t hbridge8;
static log_t logger;
uint8_t mode;

void application_init ( void ) 
{
    log_cfg_t log_cfg;            /**< Logger config object. */
    hbridge8_cfg_t hbridge8_cfg;  /**< Click config object. */

    // Logger initialization.

    LOG_MAP_USB_UART( log_cfg );
    log_cfg.level = LOG_LEVEL_DEBUG;
    log_cfg.baud = 115200;
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.

    hbridge8_cfg_setup( &hbridge8_cfg );
    HBRIDGE8_MAP_MIKROBUS( hbridge8_cfg, MIKROBUS_1 );
    err_t init_flag  = hbridge8_init( &hbridge8, &hbridge8_cfg );
    if ( PWM_ERROR == init_flag )
    {
        log_error( &logger, " Application Init Error. " );
        log_info( &logger, " Please, run program again... " );

        for ( ; ; );
    }

    hbridge8_default_cfg ( &hbridge8 );
    Delay_ms( 100 );
    
    hbridge8_set_duty_cycle ( &hbridge8, 0.0 );
    hbridge8_pwm_start( &hbridge8 );

    log_info( &logger, " Application Task " );
    mode = HBRIDGE8_MODE_FOREWORD;
    hbridge8_set_mode( &hbridge8, mode );
    log_printf( &logger, "\r\n>>> Foreword\r\n\r\n" );
    Delay_ms( 100 );
}

void application_task ( void ) 
{
    static int8_t duty_cnt = 1;
    static int8_t duty_inc = 1;
    float duty = duty_cnt / 10.0;
    
    hbridge8_set_duty_cycle ( &hbridge8, duty );
    log_printf( &logger, "> Duty: %d%%\r\n", ( uint16_t )( duty_cnt * 10 ) );
    
    Delay_ms( 500 );
    
    if ( 10 == duty_cnt ) 
    {
        duty_inc = -1;
    }
    else if ( 0 == duty_cnt ) 
    {
        duty_inc = 1;
        if ( mode == HBRIDGE8_MODE_REVERSE ) {
            mode = HBRIDGE8_MODE_FOREWORD;
            hbridge8_set_mode( &hbridge8, mode );
            log_printf( &logger, "\r\n>>> Foreword\r\n\r\n" );
        } else {
            mode = HBRIDGE8_MODE_REVERSE;
            hbridge8_set_mode( &hbridge8, mode );  
            log_printf( &logger, "\r\n>>> Reverse\r\n\r\n" );
        }

        
    }
    duty_cnt += duty_inc;
}

void main ( void )  
{
    application_init( );

    for ( ; ; ) 
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
